import '@rhtml/renderer';
import { CustomElementConfig, LitElement, TemplateResult } from '@rxdi/lit-html';
import { Observable } from 'rxjs';
export type Without<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
export type Injection = {
    has: <T>(d: T) => boolean;
    get: <T>(d: T) => T;
};
export type Constructor<T = any> = new (...args: any[]) => T;
export type InstanceTypes<T> = {
    [P in keyof T]: T[P] extends Constructor<infer U> ? U : never;
};
export declare const DefineDependencies: <T extends Constructor<any>[]>(...deps: T) => (injection: Injection) => InstanceTypes<T>;
export type StateToRender<S, D, K extends LitElement> = (this: K, deps: D) => Observable<S> | S | Promise<S>;
export type PossibleRender = TemplateResult | string | number | unknown;
export type RenderResult<S, D, K extends LitElement> = (deps: D) => (this: K, state: S, setState: (s: S) => void) => TemplateResult | string | number | unknown;
export interface Options extends Without<CustomElementConfig<never>, 'template'> {
    deepCloneState?: boolean;
}
export declare const Partial: <S, D, K extends LitElement = LitElement>(options: Options) => (deps?: D) => (state?: StateToRender<S, D, K>) => (loading: () => PossibleRender) => (error: (e: Error | unknown) => PossibleRender) => (render?: RenderResult<S, D, K>) => <K_1 extends new (...args: any[]) => {}>(Base: K_1) => {
    new (...args: any[]): {
        getTemplateResult(): any;
        OnInit(): any;
        disconnectedCallback(): void;
        connectedCallback(): void;
        render(): any;
        update(): void;
        firstUpdated(): void;
    };
    styles: import("@rxdi/lit-html").CSSResult[];
    is(): string;
} & K_1;
export declare function Component<S, D, K extends LitElement = LitElement>({ Settings, Providers, State, Render, Loading, Error, }: {
    Settings: Options;
    Providers?: D;
    State?: StateToRender<S, D, K>;
    Render?: RenderResult<S, D, K>;
    Loading?: () => PossibleRender;
    Error?: (e: Error | unknown) => PossibleRender;
}): <K_1 extends new (...args: any[]) => {}>(Base: K_1) => {
    new (...args: any[]): {
        getTemplateResult(): any;
        OnInit(): any;
        disconnectedCallback(): void;
        connectedCallback(): void;
        render(): any;
        update(): void;
        firstUpdated(): void;
    };
    styles: import("@rxdi/lit-html").CSSResult[];
    is(): string;
} & K_1;
